package javax.swing.plaf.nimbus;

import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.image.ColorModel;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.image.BufferedImage;

class EffectUtils
{
    EffectUtils() {
    }
    
    static void clearImage(BufferedImage bufferedImage) {
        Graphics2D graphics = bufferedImage.createGraphics();
        graphics.setComposite(AlphaComposite.Clear);
        graphics.fillRect(0, 0, bufferedImage.getWidth(), bufferedImage.getHeight());
        graphics.dispose();
    }
    
    static BufferedImage gaussianBlur(BufferedImage bufferedImage, BufferedImage colorModelCompatibleImage, int n) {
        int width = bufferedImage.getWidth();
        int height = bufferedImage.getHeight();
        if (colorModelCompatibleImage == null || colorModelCompatibleImage.getWidth() != width || colorModelCompatibleImage.getHeight() != height || bufferedImage.getType() != colorModelCompatibleImage.getType()) {
            colorModelCompatibleImage = createColorModelCompatibleImage(bufferedImage);
        }
        float[] gaussianKernel = createGaussianKernel(n);
        if (bufferedImage.getType() == 2) {
            int[] array3 = new int[width * height];
            int[] array4 = new int[width * height];
            getPixels(bufferedImage, 0, 0, width, height, array3);
            blur(array3, array4, width, height, gaussianKernel, n);
            blur(array4, array3, height, width, gaussianKernel, n);
            setPixels(colorModelCompatibleImage, 0, 0, width, height, array3);
        }
        else if (bufferedImage.getType() == 10) {
            byte[] array = new byte[width * height];
            byte[] array2 = new byte[width * height];
            getPixels(bufferedImage, 0, 0, width, height, array);
            blur(array, array2, width, height, gaussianKernel, n);
            blur(array2, array, height, width, gaussianKernel, n);
            setPixels(colorModelCompatibleImage, 0, 0, width, height, array);
        }
        else {
            throw new IllegalArgumentException("EffectUtils.gaussianBlur() src image is not a supported type, type=[" + bufferedImage.getType() + "]");
        }
        return colorModelCompatibleImage;
    }
    
    private static void blur(int[] array, int[] array2, int n, int n2, float[] array3, int n3) {
        for (int i = 0; i < n2; ++i) {
            int n4 = i * n;
            float n8;
            float n7;
            float n6;
            float n5;
            int n12;
            int n13;
            int n14;
            int n15;
            for (int j = 0; j < n; n12 = (int)(n6 + 0.5f), n13 = (int)(n5 + 0.5f), n14 = (int)(n7 + 0.5f), n15 = (int)(n8 + 0.5f), array2[i] = (((n12 > 255) ? 255 : n12) << 24 | ((n13 > 255) ? 255 : n13) << 16 | ((n14 > 255) ? 255 : n14) << 8 | ((n15 > 255) ? 255 : n15)), i += n2, ++j) {
                n5 = (n6 = (n7 = (n8 = 0.0f)));
                int n9;
                int n10;
                float n11;
                for (int k = -n3; k <= n3; n10 = array[n4 + n9], n11 = array3[n3 + k], n6 += n11 * (float)(n10 >> 24 & 0xFF), n5 += n11 * (float)(n10 >> 16 & 0xFF), n7 += n11 * (float)(n10 >> 8 & 0xFF), n8 += n11 * (float)(n10 & 0xFF), ++k) {
                    n9 = j + k;
                    if (n9 < 0 || n9 >= n) {
                        n9 = (j + n) % n;
                    }
                }
            }
        }
    }
    
    static void blur(byte[] array, byte[] array2, int n, int n2, float[] array3, int n3) {
        for (int i = 0; i < n2; ++i) {
            int n4 = i * n;
            float n5;
            int n7;
            for (int j = 0; j < n; n7 = (int)(n5 + 0.5f), array2[i] = (byte)((n7 > 255) ? 255 : n7), i += n2, ++j) {
                n5 = 0.0f;
                int n6;
                for (int k = -n3; k <= n3; n5 += array3[n3 + k] * (float)(array[n4 + n6] & 0xFF), ++k) {
                    n6 = j + k;
                    if (n6 < 0 || n6 >= n) {
                        n6 = (j + n) % n;
                    }
                }
            }
        }
    }
    
    static float[] createGaussianKernel(int n) {
        if (n < 1) {
            throw new IllegalArgumentException("Radius must be >= 1");
        }
        float[] array = new float[n * 2 + 1];
        float n2 = (float)n / 3.0f;
        float n3 = 2.0f * n2 * n2;
        float n4 = (float)Math.sqrt((double)n3 * 3.141592653589793);
        float n5 = 0.0f;
        for (int i = -n; i <= n; ++i) {
            float n6 = (float)(i * i);
            int n7 = i + n;
            array[n7] = (float)Math.exp((double)(-n6 / n3)) / n4;
            n5 += array[n7];
        }
        for (int j = 0; j < array.length; ++j) {
            int n8 = j;
            array[n8] /= n5;
        }
        return array;
    }
    
    static byte[] getPixels(BufferedImage bufferedImage, int x, int y, int w, int h, byte[] outData) {
        if (w == 0 || h == 0) {
            return new byte[0];
        }
        if (outData == null) {
            outData = new byte[w * h];
        }
        else if (outData.length < w * h) {
            throw new IllegalArgumentException("pixels array must have a length >= w*h");
        }
        if (bufferedImage.getType() == 10) {
            return (byte[])(byte[])bufferedImage.getRaster().getDataElements(x, y, w, h, outData);
        }
        throw new IllegalArgumentException("Only type BYTE_GRAY is supported");
    }
    
    static void setPixels(BufferedImage bufferedImage, int x, int y, int w, int h, byte[] inData) {
        if (inData == null || w == 0 || h == 0) {
            return;
        }
        if (inData.length < w * h) {
            throw new IllegalArgumentException("pixels array must have a length >= w*h");
        }
        if (bufferedImage.getType() == 10) {
            bufferedImage.getRaster().setDataElements(x, y, w, h, inData);
            return;
        }
        throw new IllegalArgumentException("Only type BYTE_GRAY is supported");
    }
    
    public static int[] getPixels(BufferedImage bufferedImage, int n, int n2, int w, int n3, int[] array) {
        if (w == 0 || n3 == 0) {
            return new int[0];
        }
        if (array == null) {
            array = new int[w * n3];
        }
        else if (array.length < w * n3) {
            throw new IllegalArgumentException("pixels array must have a length >= w*h");
        }
        int type = bufferedImage.getType();
        if (type == 2 || type == 1) {
            return (int[])(int[])bufferedImage.getRaster().getDataElements(n, n2, w, n3, array);
        }
        return bufferedImage.getRGB(n, n2, w, n3, array, 0, w);
    }
    
    public static void setPixels(BufferedImage bufferedImage, int n, int n2, int w, int n3, int[] array) {
        if (array == null || w == 0 || n3 == 0) {
            return;
        }
        if (array.length < w * n3) {
            throw new IllegalArgumentException("pixels array must have a length >= w*h");
        }
        int type = bufferedImage.getType();
        if (type == 2 || type == 1) {
            bufferedImage.getRaster().setDataElements(n, n2, w, n3, array);
        }
        else {
            bufferedImage.setRGB(n, n2, w, n3, array, 0, w);
        }
        return;
    }
    
    public static BufferedImage createColorModelCompatibleImage(BufferedImage bufferedImage) {
        ColorModel colorModel = bufferedImage.getColorModel();
        return new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(bufferedImage.getWidth(), bufferedImage.getHeight()), colorModel.isAlphaPremultiplied(), null);
    }
    
    public static BufferedImage createCompatibleTranslucentImage(int n, int n2) {
        return isHeadless() ? new BufferedImage(n, n2, 2) : getGraphicsConfiguration().createCompatibleImage(n, n2, 3);
    }
    
    private static boolean isHeadless() {
        return GraphicsEnvironment.isHeadless();
    }
    
    private static GraphicsConfiguration getGraphicsConfiguration() {
        return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
    }
}
